package com.nokia.finance.tasks.jobs.car.strategy;

import com.nokia.finance.tasks.common.exception.MyRuntimeException;
import com.nokia.finance.tasks.config.JobConfig;
import com.nokia.finance.tasks.dao.car.CarStrategyDao;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVPrinter;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.io.OutputStreamWriter;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.LinkedHashMap;
import java.util.List;

/**
 * 高油耗车辆策略定时任务
 */
@Slf4j
@Service
public class CarHighFuelConsumptionJob {
    private final JobConfig jobConfig;
    private final CarStrategyDao carStrategyDao;

    CarHighFuelConsumptionJob(JobConfig jobConfig, CarStrategyDao carStrategyDao) {
        this.jobConfig = jobConfig;
        this.carStrategyDao = carStrategyDao;
    }

    /**
     * 执行任务
     */
    @Scheduled(cron = "0 8 0 15 * ?")
    @Transactional(timeout = 60, rollbackFor = Exception.class)
    public void runJob() {
        log.info("执行高油耗车辆策略定时任务");
        LocalDate localDateNow = LocalDate.now();
        LocalDate localDateEnd = localDateNow.minusMonths(1);
        LocalDate localDateStart = localDateEnd.withDayOfYear(1);
        Integer endYearMonth = Integer.valueOf(localDateEnd.format(DateTimeFormatter.ofPattern("yyyyMM")));
        Integer startYearMonth = Integer.valueOf(localDateStart.format(DateTimeFormatter.ofPattern("yyyyMM")));
        try {
            int updateCount = carStrategyDao.insertCarHighFuelConsumption(startYearMonth, endYearMonth);
            if (updateCount == 0) {
                throw new MyRuntimeException("插入高油耗车辆策略失败");
            }
            List<LinkedHashMap<String, Object>> list = carStrategyDao.getCarHighFuelConsumption(endYearMonth);
            if (CollectionUtils.isEmpty(list)) {
                throw new MyRuntimeException("高油耗车辆策略为空");
            }
            Files.createDirectories(Paths.get(jobConfig.getCarHighFuelConsumptionOutputPath()));
            char delimiter = 1;
            try (OutputStreamWriter osw = new OutputStreamWriter(
                    Files.newOutputStream(Paths.get(jobConfig.getCarHighFuelConsumptionOutputPath()
                            + "car_high_fuel_consumption_" + endYearMonth + ".csv")),
                    StandardCharsets.UTF_8);
                 CSVPrinter printer = new CSVPrinter(osw,
                         CSVFormat.DEFAULT.builder().setDelimiter(delimiter).build())) {
                // 添加bom头避免excel乱码
                osw.write('\ufeff');
                LinkedHashMap<String, Object> header = list.get(0);
                // 表头
                printer.printRecord(header.keySet());
                for (LinkedHashMap<String, Object> map : list) {
                    printer.printRecord(map.values());
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new MyRuntimeException("高油耗车辆策略定时任务失败");
        }
    }
}
