package com.nokia.finance.tasks.jobs.car.strategy;

import com.nokia.finance.tasks.common.exception.MyRuntimeException;
import com.nokia.finance.tasks.config.JobConfig;
import com.nokia.finance.tasks.dao.car.CarStrategyDao;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVPrinter;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.io.OutputStreamWriter;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * 租赁车辆产生维修费策略定时任务
 */
@Slf4j
@Service
public class CarRentalRepairJob {
    private final JobConfig jobConfig;
    private final CarStrategyDao carStrategyDao;

    CarRentalRepairJob(JobConfig jobConfig, CarStrategyDao carStrategyDao) {
        this.jobConfig = jobConfig;
        this.carStrategyDao = carStrategyDao;
    }
    /**
     * 执行任务
     */
    @Scheduled(cron = "0 4 0 15 * ?")
    @Transactional(timeout = 60, rollbackFor = Exception.class)
    public void runJob() {
        log.info("执行租赁车辆产生维修费策略定时任务");
        LocalDate localDateNow = LocalDate.now();
        LocalDate localDateEnd = localDateNow.minusMonths(1);
        LocalDate localDateStart = localDateEnd.withDayOfYear(1);
        Integer endYearMonth = Integer.valueOf(localDateEnd.format(DateTimeFormatter.ofPattern("yyyyMM")));
        Integer startYearMonth = Integer.valueOf(localDateStart.format(DateTimeFormatter.ofPattern("yyyyMM")));
        try {
            boolean flag = carStrategyDao.hasCarRentalRepair(endYearMonth);
            if (!flag) {
                throw new MyRuntimeException("租赁车辆产生维修费策略缺少数据");
            }
            carStrategyDao.insertCarRentalRepair(startYearMonth, endYearMonth);
            List<LinkedHashMap<String, Object>> list = carStrategyDao.getCarRentalRepair(endYearMonth);
            Files.createDirectories(Paths.get(jobConfig.getCarRentalRepairOutputPath()));
            char delimiter = 1;
            try (OutputStreamWriter osw = new OutputStreamWriter(
                    Files.newOutputStream(Paths.get(jobConfig.getCarRentalRepairOutputPath()
                            + "car_rental_repair_" + endYearMonth + ".csv")),
                    StandardCharsets.UTF_8);
                 CSVPrinter printer = new CSVPrinter(osw,
                         CSVFormat.DEFAULT.builder().setDelimiter(delimiter).build())) {
                if (CollectionUtils.isEmpty(list)) {
                    return;
                }
                // 添加bom头避免excel乱码
                osw.write('\ufeff');
                Map<String, Object> header = list.get(0);
                // 表头
                printer.printRecord(header.keySet());
                for (Map<String, Object> map : list) {
                    printer.printRecord(map.values());
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new MyRuntimeException("租赁车辆产生维修费策略定时任务失败");
        }
    }
}
