package com.nokia.finance.tasks.jobs.house.fcdp;

import com.nokia.finance.tasks.common.exception.MyRuntimeException;
import com.nokia.finance.tasks.config.JobConfig;
import com.nokia.finance.tasks.dao.house.HouseFcdpDao;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVPrinter;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.io.OutputStreamWriter;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * 房产大屏建筑和土地数量统计定时任务
 */
@Slf4j
@Service
public class HouseBuildingLandCountJob {
    private final JobConfig jobConfig;
    private final HouseFcdpDao houseFcdpDao;

    HouseBuildingLandCountJob(JobConfig jobConfig, HouseFcdpDao houseFcdpDao) {
        this.jobConfig = jobConfig;
        this.houseFcdpDao = houseFcdpDao;
    }
    /**
     * 执行任务
     */
    @Scheduled(cron = "0 2 0 10 * ?")
    @Transactional(timeout = 60, rollbackFor = Exception.class)
    public void runJob() {
        log.info("执行房产大屏建筑和土地数量统计定时任务");
        LocalDate localDateNow = LocalDate.now();
        LocalDate localDateEnd = localDateNow.minusMonths(1);
        Integer endYearMonth = Integer.valueOf(localDateEnd.format(DateTimeFormatter.ofPattern("yyyyMM")));
        try {
            int update1 = houseFcdpDao.insertHouseBuildingLandCount(endYearMonth);
            if (update1 == 0) {
                throw new MyRuntimeException("房产大屏建筑和土地数量统计定时任务失败");
            }
            List<LinkedHashMap<String, Object>> list = houseFcdpDao.getHouseBuildingLandCount(endYearMonth);
            Files.createDirectories(Paths.get(jobConfig.getHouseBuildingLandCountOutputPath()));
            char delimiter = 1;
            try (OutputStreamWriter osw = new OutputStreamWriter(
                    Files.newOutputStream(Paths.get(jobConfig.getHouseBuildingLandCountOutputPath()
                            + "building_land_count_" + endYearMonth + ".csv")),
                    StandardCharsets.UTF_8);
                 CSVPrinter printer = new CSVPrinter(osw,
                         CSVFormat.DEFAULT.builder().setDelimiter(delimiter).build())) {
                if (CollectionUtils.isEmpty(list)) {
                    return;
                }
                // 添加bom头避免excel乱码
                osw.write('\ufeff');
                Map<String, Object> header = list.get(0);
                // 表头
                printer.printRecord(header.keySet());
                for (Map<String, Object> map : list) {
                    printer.printRecord(map.values());
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new MyRuntimeException("房产大屏建筑和土地数量统计定时任务失败");
        }
    }
}
